'use strict';

var fastStringify = require('fast-safe-stringify');
var quickFormat = require('./quick-format.cjs');

/**
 * @typedef {object} FormatOption
 * @property {number} [spaces] number of spaces to use for formatting
 */

class Format {
  /**
   * @param {FormatOption} [opts]
   */
  constructor (opts) {
    const { spaces } = opts || {};
    this.opts = { spaces };
    this._formatOpts();
  }

  get spaces () {
    return this.opts.spaces
  }

  set spaces (spaces) {
    this.opts.spaces = spaces;
    this._formatOpts();
  }

  _formatOpts () {
    this.formatOpts = {
      // @ts-expect-error
      stringify: (o) => fastStringify(o, null, this.opts.spaces),
      spaces: this.opts.spaces
    };
  }

  stringify (...args) {
    // @ts-expect-error
    return fastStringify(...args)
  }

  /**
   * formats arguments like `util.format`
   * @param {any} fmt may contain "%" formatters
   * @param {any} args arguments list
   * @param {any} obj
   * @return {Array} first is formatted message, other args may follow
   */
  format (fmt, args, obj) {
    return quickFormat.format(fmt, args, this.formatOpts, obj)
  }
}

exports.Format = Format;
